@@ -30,7 +30,6 @@ PERFORMANCE OF THIS SOFTWARE.
 #include <config.h>
 #endif
 #include	<X11/Xatom.h>
-#define NEED_EVENTS
 #include	"Xlibint.h"
 #include	"Xlcint.h"
 #include	"XlcPublic.h"
@@ -53,11 +52,11 @@ typedef struct _XimInstCallback {
 } XimInstCallbackRec, *XimInstCallback;
 
 
-Private XimInstCallback	callback_list	= NULL;
-Private Bool		lock		= False;
+static XimInstCallback	callback_list	= NULL;
+static Bool		lock		= False;
 
 
-Private void
+static void
 MakeLocale( XLCd lcd, char locale[] )
 {
     char	*language, *territory, *codeset;
@@ -77,7 +76,7 @@ MakeLocale( XLCd lcd, char locale[] )
 }
 
 
-Private Bool
+static Bool
 _XimFilterPropertyNotify(
     Display	*display,
     Window	 window,
@@ -109,7 +108,7 @@ _XimFilterPropertyNotify(
     }
 
     lock = True;
-    for( ii = 0; ii < nitems; ii++, atoms ) {
+    for( ii = 0; ii < nitems; ii++ ) {
 	if(XGetSelectionOwner (display, atoms[ii])) {
 	    for( icb = callback_list; icb; icb = icb->next ) {
 		if( !icb->call  &&  !icb->destroy ) {
@@ -151,7 +150,7 @@ _XimFilterPropertyNotify(
 }
 
 
-Public Bool
+Bool
 _XimRegisterIMInstantiateCallback(
     XLCd	 lcd,
     Display	*display,
@@ -163,20 +162,28 @@ _XimRegisterIMInstantiateCallback(
 {
     XimInstCallback	icb, tmp;
     XIM			xim;
+    char		*modifiers = NULL;
     Window		root;
     XWindowAttributes	attr;
 
     if( lock )
 	return( False );
 
-    icb = (XimInstCallback)Xmalloc(sizeof(XimInstCallbackRec));
+    icb = Xmalloc(sizeof(XimInstCallbackRec));
     if( !icb )
 	return( False );
+    if (lcd->core->modifiers) {
+	modifiers = strdup(lcd->core->modifiers);
+	if (!modifiers) {
+	    Xfree(icb);
+	    return( False );
+	}
+    }
     icb->call = icb->destroy = False;
     icb->display = display;
     icb->lcd = lcd;
     MakeLocale( lcd, icb->name );
-    icb->modifiers = lcd->core->modifiers;	/* XXXXX */
+    icb->modifiers = modifiers;
     icb->rdb = rdb;
     icb->res_name = res_name;
     icb->res_class = res_class;
@@ -214,7 +221,7 @@ _XimRegisterIMInstantiateCallback(
 }
 
 
-Public Bool
+Bool
 _XimUnRegisterIMInstantiateCallback(
     XLCd	 lcd,
     Display	*display,
@@ -259,6 +266,7 @@ _XimUnRegisterIMInstantiateCallback(
 		else
 		    picb->next = icb->next;
 		_XCloseLC( icb->lcd );
+		XFree( icb->modifiers );
 		XFree( icb );
 	    }
 	    return( True );
@@ -268,7 +276,7 @@ _XimUnRegisterIMInstantiateCallback(
 }
 
 
-Public void
+void
 _XimResetIMInstantiateCallback(Xim xim)
 {
     char		 locale[XIM_MAXLCNAMELEN];
