@@ -1,4 +1,3 @@
-/* $Xorg: lcGenConv.c,v 1.5 2000/08/17 19:45:17 cpqbld Exp $ */
 /*
  * Copyright 1992, 1993 by TOSHIBA Corp.
  *
@@ -35,7 +34,6 @@
  *  2000
  *  Modifier: Ivan Pascal      The XFree86 Project
  */
-/* $XFree86: xc/lib/X11/lcGenConv.c,v 3.27tsi Exp $ */
 
 /*
  * A generic locale loader for all kinds of ISO-2022 based codesets.
@@ -344,7 +342,7 @@ byteM_parse_codeset(
 	    continue;
 
         for (j = 0; j < codeset->length; j++) {
-	    ch = *((unsigned char *)(inbufptr + j));
+	    ch = *((const unsigned char *)(inbufptr + j));
 	    byteM_rec = byteM[j];
 	    byteinfo = byteM_rec.byteinfo;
 
@@ -755,7 +753,7 @@ mbstowcs_org(
 
     CodeSet codeset = NULL;
 
-    const char *inbufptr = *from;
+    const char *inbufptr;
     wchar_t *outbufptr = (wchar_t *) *to;
     int from_size = *from_left;
 
@@ -766,8 +764,7 @@ mbstowcs_org(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
+    inbufptr = *from;
 
     while (*from_left && *to_left) {
 
@@ -923,9 +920,6 @@ wcstombs_org(
     int defstr_len = strlen(default_string);
 
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -946,7 +940,7 @@ wcstombs_org(
             if (*to_left < defstr_len)
 		break;
 	    if (outbufptr) {
-                strncpy((char *)outbufptr, default_string, defstr_len);
+                memcpy(outbufptr, default_string, defstr_len);
                 outbufptr += defstr_len;
             }
 	    (*to_left) -= defstr_len;
@@ -982,7 +976,7 @@ wcstombs_org(
                     if (*to_left < length)
 		        break;
 	            if (outbufptr) {
-                        strncpy((char *)outbufptr, encoding, length);
+			memcpy(outbufptr, encoding, length);
 	                outbufptr += length;
                     }
 	            (*to_left) -= length;
@@ -1082,9 +1076,6 @@ wcstocts(
     int from_size = *from_left;
     char *ext_seg_len = NULL;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -1234,8 +1225,7 @@ stdc_wcstocts(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1263,7 +1253,7 @@ ctstowcs(
     CodeSet codeset = NULL;
     XlcCharSet charset_tmp;
 
-    const char *inbufptr = *from;
+    const char *inbufptr;
     wchar_t *outbufptr = (wchar_t *) *to;
     int from_size = *from_left;
 
@@ -1273,9 +1263,7 @@ ctstowcs(
 	_XlcResetConverter(conv);
         return( 0 );
     }
-
-    if (*from_left > *to_left)
-        *from_left = *to_left;
+    inbufptr = *from;
 
     while (*from_left && *to_left) {
 
@@ -1451,7 +1439,7 @@ cstowcs(
     CodeSet codeset = NULL;
     XlcCharSet charset, charset_tmp;
 
-    const char *inbufptr = *from;
+    const char *inbufptr;
     wchar_t *outbufptr = (wchar_t *) *to;
     int from_size = *from_left;
 
@@ -1459,6 +1447,8 @@ cstowcs(
         return( 0 );
     }
 
+    inbufptr = *from;
+
     charset = (XlcCharSet) args[0];
 
     while (*from_left && *to_left) {
@@ -1557,8 +1547,7 @@ stdc_ctstowcs(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1593,8 +1582,7 @@ stdc_cstowcs(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1629,8 +1617,7 @@ mbstocts(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1657,7 +1644,7 @@ mbstostr(
 
     CodeSet codeset = NULL;
 
-    const char *inbufptr = *from;
+    const char *inbufptr;
     char *outbufptr = *to;
     int from_size = *from_left;
 
@@ -1668,8 +1655,7 @@ mbstostr(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
+    inbufptr = *from;
 
     while (*from_left && *to_left) {
 
@@ -1775,7 +1761,7 @@ mbtocs(
     CodeSet codeset = NULL;
     XlcCharSet charset = NULL;
 
-    const char *inbufptr = *from;
+    const char *inbufptr;
     char *outbufptr = *to;
     int from_size = *from_left;
 
@@ -1786,8 +1772,7 @@ mbtocs(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
+    inbufptr = *from;
 
     while (*from_left && *to_left) {
 
@@ -1874,7 +1859,7 @@ output:
 	return -1;
     }
 
-    /* nomal end */
+    /* normal end */
     *from = (XPointer) inbufptr;
     *to = (XPointer) outbufptr;
 
@@ -1961,10 +1946,6 @@ wcstostr(
     const char *default_string = XLC_PUBLIC(lcd, default_string);
     int defstr_len = strlen(default_string);
 
-
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -1985,7 +1966,7 @@ wcstostr(
             if (*to_left < defstr_len)
 		break;
 	    if (outbufptr) {
-                strncpy((char *)outbufptr, default_string, defstr_len);
+		memcpy(outbufptr, default_string, defstr_len);
 	        outbufptr += defstr_len;
             }
 	    (*to_left) -= defstr_len;
@@ -2024,7 +2005,7 @@ wcstostr(
 		            break;
 
 	                if (outbufptr) {
-                            strncpy((char *)outbufptr, encoding, length);
+			    memcpy(outbufptr, encoding, length);
 	                    outbufptr += length;
                         }
 	                (*to_left) -= length;
@@ -2086,8 +2067,7 @@ stdc_wcstostr(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2119,9 +2099,6 @@ wctocs(
     char *outbufptr = *to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     if (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -2173,7 +2150,7 @@ end:
         return -1;
     }
 
-    /* nomal end */
+    /* normal end */
     *from = (XPointer) inbufptr;
     *to = (XPointer) outbufptr;
 
@@ -2227,7 +2204,7 @@ end:
 	return -1;
     }
 
-    /* nomal end */
+    /* normal end */
     *from = (XPointer) src;
     *from_left = src_left;
 
@@ -2364,8 +2341,7 @@ ctstombs(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2400,8 +2376,7 @@ cstombs(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2432,9 +2407,6 @@ strtombs(
     char *outbufptr = *to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         ch = *inbufptr++;
@@ -2490,7 +2462,7 @@ strtombs(
                 if (*to_left < length)
 		    break;
 	        if (outbufptr) {
-                    strncpy((char *)outbufptr, encoding, length);
+		    memcpy(outbufptr, encoding, length);
 	            outbufptr += length;
                 }
 	        (*to_left) -= length;
@@ -2542,9 +2514,6 @@ strtowcs(
     wchar_t *outbufptr = (wchar_t *)*to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         ch = *inbufptr++;
@@ -2615,8 +2584,7 @@ stdc_strtowcs(
         goto ret;
 
 ret:
-    if (buf)
-	Xfree((char *)buf);
+    Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2629,15 +2597,9 @@ static void
 close_converter(
     XlcConv conv)
 {
-    if (conv->state) {
-	Xfree((char *) conv->state);
-    }
-
-    if (conv->methods) {
-	Xfree((char *) conv->methods);
-    }
-
-    Xfree((char *) conv);
+    Xfree(conv->state);
+    Xfree(conv->methods);
+    Xfree(conv);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -2652,20 +2614,19 @@ create_conv(
     XlcConv conv;
     State state;
 
-    conv = (XlcConv) Xmalloc(sizeof(XlcConvRec));
+    conv = Xcalloc(1, sizeof(XlcConvRec));
     if (conv == NULL)
 	return (XlcConv) NULL;
 
-    conv->methods = (XlcConvMethods) Xmalloc(sizeof(XlcConvMethodsRec));
+    conv->methods = Xmalloc(sizeof(XlcConvMethodsRec));
     if (conv->methods == NULL)
 	goto err;
     *conv->methods = *methods;
     conv->methods->reset = init_state;
 
-    conv->state = (XPointer) Xmalloc(sizeof(StateRec));
+    conv->state = Xcalloc(1, sizeof(StateRec));
     if (conv->state == NULL)
 	goto err;
-    bzero((char *) conv->state, sizeof(StateRec));
 
     state = (State) conv->state;
     state->lcd = lcd;
