@@ -19,6 +19,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
@@ -54,7 +55,7 @@ static void return_socket(void *closure)
 	UnlockDisplay(dpy);
 }
 
-static void require_socket(Display *dpy)
+static Bool require_socket(Display *dpy)
 {
 	if(dpy->bufmax == dpy->buffer)
 	{
@@ -65,26 +66,15 @@ static void require_socket(Display *dpy)
 		if(dpy->xcb->event_owner != XlibOwnsEventQueue)
 			flags = XCB_REQUEST_CHECKED;
 		if(!xcb_take_socket(dpy->xcb->connection, return_socket, dpy,
-		                    flags, &sent))
+		                    flags, &sent)) {
 			_XIOError(dpy);
-		/* Xlib uses unsigned long for sequence numbers.  XCB
-		 * uses 64-bit internally, but currently exposes an
-		 * unsigned int API.  If these differ, Xlib cannot track
-		 * the full 64-bit sequence number if 32-bit wrap
-		 * happens while Xlib does not own the socket.  A
-		 * complete fix would be to make XCB's public API use
-		 * 64-bit sequence numbers. */
-		if (sizeof(unsigned long) > sizeof(unsigned int) &&
-		    dpy->xcb->event_owner == XlibOwnsEventQueue &&
-		    (sent - dpy->last_request_read >= (UINT64_C(1) << 32))) {
-			throw_thread_fail_assert("Sequence number wrapped "
-			                         "beyond 32 bits while Xlib "
-						 "did not own the socket",
-			                         xcb_xlib_seq_number_wrapped);
+			return False;
 		}
-		dpy->xcb->last_flushed = dpy->request = sent;
+		dpy->xcb->last_flushed = sent;
+		X_DPY_SET_REQUEST(dpy, sent);
 		dpy->bufmax = dpy->xcb->real_bufmax;
 	}
+	return True;
 }
 
 /* Call internal connection callbacks for any fds that are currently
@@ -104,7 +94,7 @@ static void require_socket(Display *dpy)
  *  _XEatData
  * _XReadPad
  */
-static void check_internal_connections(Display *dpy)
+static Bool check_internal_connections(Display *dpy)
 {
 	struct _XConnectionInfo *ilist;
 	fd_set r_mask;
@@ -113,7 +103,7 @@ static void check_internal_connections(Display *dpy)
 	int highest_fd = -1;
 
 	if(dpy->flags & XlibDisplayProcConni || !dpy->im_fd_info)
-		return;
+		return True;
 
 	FD_ZERO(&r_mask);
 	for(ilist = dpy->im_fd_info; ilist; ilist = ilist->next)
@@ -131,9 +121,12 @@ static void check_internal_connections(Display *dpy)
 
 	if(result == -1)
 	{
-		if(errno == EINTR)
-			return;
-		_XIOError(dpy);
+		if(errno != EINTR) {
+			_XIOError(dpy);
+			return False;
+		}
+
+		return True;
 	}
 
 	for(ilist = dpy->im_fd_info; result && ilist; ilist = ilist->next)
@@ -142,9 +135,11 @@ static void check_internal_connections(Display *dpy)
 			_XProcessInternalConnection(dpy, ilist);
 			--result;
 		}
+
+	return True;
 }
 
-static PendingRequest *append_pending_request(Display *dpy, unsigned long sequence)
+static PendingRequest *append_pending_request(Display *dpy, uint64_t sequence)
 {
 	PendingRequest *node = malloc(sizeof(PendingRequest));
 	assert(node);
@@ -213,14 +208,13 @@ static int handle_error(Display *dpy, xError *err, Bool in_XReply)
 	return 0;
 }
 
-/* Widen a 32-bit sequence number into a native-word-size (unsigned long)
- * sequence number.  Treating the comparison as a 1 and shifting it avoids a
- * conditional branch, and shifting by 16 twice avoids a compiler warning when
- * sizeof(unsigned long) == 4. */
-static void widen(unsigned long *wide, unsigned int narrow)
+/* Widen a 32-bit sequence number into a 64bit (uint64_t) sequence number.
+ * Treating the comparison as a 1 and shifting it avoids a conditional branch.
+ */
+static void widen(uint64_t *wide, unsigned int narrow)
 {
-	unsigned long new = (*wide & ~0xFFFFFFFFUL) | narrow;
-	*wide = new + ((unsigned long) (new < *wide) << 16 << 16);
+	uint64_t new = (*wide & ~((uint64_t)0xFFFFFFFFUL)) | narrow;
+	*wide = new + (((uint64_t)(new < *wide)) << 32);
 }
 
 /* Thread-safety rules:
@@ -244,35 +238,40 @@ static void widen(unsigned long *wide, unsigned int narrow)
  * variable for that thread to process the response and wake us up.
  */
 
-static xcb_generic_reply_t *poll_for_event(Display *dpy)
+static xcb_generic_reply_t *poll_for_event(Display *dpy, Bool queued_only)
 {
 	/* Make sure the Display's sequence numbers are valid */
-	require_socket(dpy);
+	if (!require_socket(dpy))
+		return NULL;
 
 	/* Precondition: This thread can safely get events from XCB. */
 	assert(dpy->xcb->event_owner == XlibOwnsEventQueue && !dpy->xcb->event_waiter);
 
-	if(!dpy->xcb->next_event)
-		dpy->xcb->next_event = xcb_poll_for_event(dpy->xcb->connection);
+	if(!dpy->xcb->next_event) {
+		if(queued_only)
+			dpy->xcb->next_event = xcb_poll_for_queued_event(dpy->xcb->connection);
+		else
+			dpy->xcb->next_event = xcb_poll_for_event(dpy->xcb->connection);
+	}
 
 	if(dpy->xcb->next_event)
 	{
 		PendingRequest *req = dpy->xcb->pending_requests;
 		xcb_generic_event_t *event = dpy->xcb->next_event;
-		unsigned long event_sequence = dpy->last_request_read;
+		uint64_t event_sequence = X_DPY_GET_LAST_REQUEST_READ(dpy);
 		widen(&event_sequence, event->full_sequence);
 		if(!req || XLIB_SEQUENCE_COMPARE(event_sequence, <, req->sequence)
 		        || (event->response_type != X_Error && event_sequence == req->sequence))
 		{
-			if (XLIB_SEQUENCE_COMPARE(event_sequence, >,
-			                          dpy->request))
+			uint64_t request = X_DPY_GET_REQUEST(dpy);
+			if (XLIB_SEQUENCE_COMPARE(event_sequence, >, request))
 			{
 				throw_thread_fail_assert("Unknown sequence "
 				                         "number while "
 							 "processing queue",
 				                xcb_xlib_threads_sequence_lost);
 			}
-			dpy->last_request_read = event_sequence;
+			X_DPY_SET_LAST_REQUEST_READ(dpy, event_sequence);
 			dpy->xcb->next_event = NULL;
 			return (xcb_generic_reply_t *) event;
 		}
@@ -283,20 +282,91 @@ static xcb_generic_reply_t *poll_for_event(Display *dpy)
 static xcb_generic_reply_t *poll_for_response(Display *dpy)
 {
 	void *response;
-	xcb_generic_error_t *error;
+	xcb_generic_reply_t *event;
 	PendingRequest *req;
-	while(!(response = poll_for_event(dpy)) &&
-	      (req = dpy->xcb->pending_requests) &&
-	      !req->reply_waiter &&
-	      xcb_poll_for_reply(dpy->xcb->connection, req->sequence, &response, &error))
+
+	while(1)
 	{
-		if(XLIB_SEQUENCE_COMPARE(req->sequence, >, dpy->request))
+		xcb_generic_error_t *error = NULL;
+		uint64_t request;
+		Bool poll_queued_only = dpy->xcb->next_response != NULL;
+
+		/* Step 1: is there an event in our queue before the next
+		 * reply/error? Return that first.
+		 *
+		 * If we don't have a reply/error saved from an earlier
+		 * invocation we check incoming events too, otherwise only
+		 * the ones already queued.
+		 */
+		response = poll_for_event(dpy, poll_queued_only);
+		if(response)
+			break;
+
+		/* Step 2:
+		 * Response is NULL, i.e. we have no events.
+		 * If we are not waiting for a reply or some other thread
+		 * had dibs on the next reply, exit.
+		 */
+		req = dpy->xcb->pending_requests;
+		if(!req || req->reply_waiter)
+			break;
+
+		/* Step 3:
+		 * We have some response (error or reply) related to req
+		 * saved from an earlier invocation of this function. Let's
+		 * use that one.
+		 */
+		if(dpy->xcb->next_response)
+		{
+			if (((xcb_generic_reply_t*)dpy->xcb->next_response)->response_type == X_Error)
+			{
+				error = dpy->xcb->next_response;
+				response = NULL;
+			}
+			else
+			{
+				response = dpy->xcb->next_response;
+				error = NULL;
+			}
+			dpy->xcb->next_response = NULL;
+		}
+		else
+		{
+			/* Step 4: pull down the next response from the wire. This
+			 * should be the 99% case.
+			 * xcb_poll_for_reply64() may also pull down events that
+			 * happened before the reply.
+			 */
+			if(!xcb_poll_for_reply64(dpy->xcb->connection, req->sequence,
+						 &response, &error)) {
+				/* if there is no reply/error, xcb_poll_for_reply64
+				 * may have read events. Return that. */
+				response = poll_for_event(dpy, True);
+				break;
+			}
+
+			/* Step 5: we have a new response, but we may also have some
+			 * events that happened before that response. Return those
+			 * first and save our reply/error for the next invocation.
+			 */
+			event = poll_for_event(dpy, True);
+			if(event)
+			{
+				dpy->xcb->next_response = error ? error : response;
+				response = event;
+				break;
+			}
+		}
+
+		/* Step 6: actually handle the reply/error now... */
+		request = X_DPY_GET_REQUEST(dpy);
+		if(XLIB_SEQUENCE_COMPARE(req->sequence, >, request))
 		{
 			throw_thread_fail_assert("Unknown sequence number "
 			                         "while awaiting reply",
 			                        xcb_xlib_threads_sequence_lost);
 		}
-		dpy->last_request_read = req->sequence;
+		X_DPY_SET_LAST_REQUEST_READ(dpy, req->sequence);
 		if(response)
 			break;
 		dequeue_pending_request(dpy, req);
@@ -351,8 +421,8 @@ int _XEventsQueued(Display *dpy, int mode)
 
 	if(mode == QueuedAfterFlush)
 		_XSend(dpy, NULL, 0);
-	else
-		check_internal_connections(dpy);
+	else if (!check_internal_connections(dpy))
+		return 0;
 
 	/* If another thread is blocked waiting for events, then we must
 	 * let that thread pick up the next event. Since it blocked, we
@@ -361,8 +431,10 @@ int _XEventsQueued(Display *dpy, int mode)
 	{
 		while((response = poll_for_response(dpy)))
 			handle_response(dpy, response, False);
-		if(xcb_connection_has_error(dpy->xcb->connection))
+		if(xcb_connection_has_error(dpy->xcb->connection)) {
 			_XIOError(dpy);
+			return 0;
+		}
 	}
 	return dpy->qlen;
 }
@@ -380,7 +452,8 @@ void _XReadEvents(Display *dpy)
 	_XSend(dpy, NULL, 0);
 	if(dpy->xcb->event_owner != XlibOwnsEventQueue)
 		return;
-	check_internal_connections(dpy);
+	if (!check_internal_connections(dpy))
+		return;
 
 	serial = dpy->next_event_serial_num;
 	while(serial == dpy->next_event_serial_num || dpy->qlen == 0)
@@ -410,7 +483,10 @@ void _XReadEvents(Display *dpy)
 			dpy->xcb->event_waiter = 0;
 			ConditionBroadcast(dpy, dpy->xcb->event_notify);
 			if(!event)
+			{
 				_XIOError(dpy);
+				return;
+			}
 			dpy->xcb->next_event = event;
 		}
 
@@ -428,7 +504,10 @@ void _XReadEvents(Display *dpy)
 			ConditionWait(dpy, dpy->xcb->reply_notify);
 		}
 		else
-			_XIOError(dpy);
+		{
+		        _XIOError(dpy);
+		        return;
+		}
 	}
 
 	/* The preceding loop established that there is no
@@ -455,6 +534,7 @@ void _XSend(Display *dpy, const char *data, long size)
 	static char const pad[3];
 	struct iovec vec[3];
 	uint64_t requests;
+	uint64_t dpy_request;
 	_XExtension *ext;
 	xcb_connection_t *c = dpy->xcb->connection;
 	if(dpy->flags & XlibDisplayIOError)
@@ -463,6 +543,10 @@ void _XSend(Display *dpy, const char *data, long size)
 	if(dpy->bufptr == dpy->buffer && !size)
 		return;
 
+	/* append_pending_request does not alter the dpy request number
+	 * therefore we can get it outside of the loop and the if
+	 */
+	dpy_request = X_DPY_GET_REQUEST(dpy);
 	/* iff we asked XCB to set aside errors, we must pick those up
 	 * eventually. iff there are async handlers, we may have just
 	 * issued requests that will generate replies. in either case,
@@ -470,11 +554,11 @@ void _XSend(Display *dpy, const char *data, long size)
 	if(dpy->xcb->event_owner != XlibOwnsEventQueue || dpy->async_handlers)
 	{
 		uint64_t sequence;
-		for(sequence = dpy->xcb->last_flushed + 1; sequence <= dpy->request; ++sequence)
+		for(sequence = dpy->xcb->last_flushed + 1; sequence <= dpy_request; ++sequence)
 			append_pending_request(dpy, sequence);
 	}
-	requests = dpy->request - dpy->xcb->last_flushed;
-	dpy->xcb->last_flushed = dpy->request;
+	requests = dpy_request - dpy->xcb->last_flushed;
+	dpy->xcb->last_flushed = dpy_request;
 
 	vec[0].iov_base = dpy->buffer;
 	vec[0].iov_len = dpy->bufptr - dpy->buffer;
@@ -491,12 +575,15 @@ void _XSend(Display *dpy, const char *data, long size)
 				ext->before_flush(dpy, &ext->codes, vec[i].iov_base, vec[i].iov_len);
 	}
 
-	if(xcb_writev(c, vec, 3, requests) < 0)
+	if(xcb_writev(c, vec, 3, requests) < 0) {
 		_XIOError(dpy);
+		return;
+	}
 	dpy->bufptr = dpy->buffer;
 	dpy->last_req = (char *) &dummy_request;
 
-	check_internal_connections(dpy);
+	if (!check_internal_connections(dpy))
+		return;
 
 	_XSetSeqSyncFunction(dpy);
 }
@@ -507,7 +594,9 @@ void _XSend(Display *dpy, const char *data, long size)
  */
 void _XFlush(Display *dpy)
 {
-	require_socket(dpy);
+	if (!require_socket(dpy))
+		return;
+
 	_XSend(dpy, NULL, 0);
 
 	_XEventsQueued(dpy, QueuedAfterReading);
@@ -569,6 +658,7 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 	xcb_connection_t *c = dpy->xcb->connection;
 	char *reply;
 	PendingRequest *current;
+	uint64_t dpy_request;
 
 	if (dpy->xcb->reply_data)
 		throw_extlib_fail_assert("Extra reply data still left in queue",
@@ -578,10 +668,12 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 		return 0;
 
 	_XSend(dpy, NULL, 0);
-	if(dpy->xcb->pending_requests_tail && dpy->xcb->pending_requests_tail->sequence == dpy->request)
+	dpy_request = X_DPY_GET_REQUEST(dpy);
+	if(dpy->xcb->pending_requests_tail
+	   && dpy->xcb->pending_requests_tail->sequence == dpy_request)
 		current = dpy->xcb->pending_requests_tail;
 	else
-		current = append_pending_request(dpy, dpy->request);
+		current = append_pending_request(dpy, dpy_request);
 	/* Don't let any other thread get this reply. */
 	current->reply_waiter = 1;
 
@@ -598,9 +690,9 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 		}
 		req->reply_waiter = 1;
 		UnlockDisplay(dpy);
-		response = xcb_wait_for_reply(c, req->sequence, &error);
+		response = xcb_wait_for_reply64(c, req->sequence, &error);
 		/* Any user locks on another thread must have been taken
-		 * while we slept in xcb_wait_for_reply. Classic Xlib
+		 * while we slept in xcb_wait_for_reply64. Classic Xlib
 		 * ignored those user locks in this case, so we do too. */
 		InternalLockDisplay(dpy, /* ignore user locks */ 1);
 
@@ -612,28 +704,25 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 		if(dpy->xcb->event_owner == XlibOwnsEventQueue)
 		{
 			xcb_generic_reply_t *event;
-			/* If some thread is already waiting for events,
-			 * it will get the first one. That thread must
-			 * process that event before we can continue. */
-			/* FIXME: That event might be after this reply,
-			 * and might never even come--or there might be
-			 * multiple threads trying to get events. */
-			while(dpy->xcb->event_waiter)
-			{ /* need braces around ConditionWait */
-				ConditionWait(dpy, dpy->xcb->event_notify);
-			}
-			while((event = poll_for_event(dpy)))
-				handle_response(dpy, event, True);
+
+			/* Assume event queue is empty if another thread is blocking
+			 * waiting for event. */
+			if(!dpy->xcb->event_waiter)
+			{
+				while((event = poll_for_response(dpy)))
+					handle_response(dpy, event, True);
+                        }
 		}
 
 		req->reply_waiter = 0;
 		ConditionBroadcast(dpy, dpy->xcb->reply_notify);
-		if(XLIB_SEQUENCE_COMPARE(req->sequence, >, dpy->request)) {
+		dpy_request = X_DPY_GET_REQUEST(dpy);
+		if(XLIB_SEQUENCE_COMPARE(req->sequence, >, dpy_request)) {
 			throw_thread_fail_assert("Unknown sequence number "
 			                         "while processing reply",
 			                        xcb_xlib_threads_sequence_lost);
 		}
-		dpy->last_request_read = req->sequence;
+		X_DPY_SET_LAST_REQUEST_READ(dpy, req->sequence);
 		if(!response)
 			dequeue_pending_request(dpy, req);
 
@@ -648,14 +737,16 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 		else if(response)
 			handle_response(dpy, response, True);
 	}
-	check_internal_connections(dpy);
+	if (!check_internal_connections(dpy))
+		return 0;
 
 	if(dpy->xcb->next_event && dpy->xcb->next_event->response_type == X_Error)
 	{
 		xcb_generic_event_t *event = dpy->xcb->next_event;
-		unsigned long event_sequence = dpy->last_request_read;
+		uint64_t last_request_read = X_DPY_GET_LAST_REQUEST_READ(dpy);
+		uint64_t event_sequence = last_request_read;
 		widen(&event_sequence, event->full_sequence);
-		if(event_sequence == dpy->last_request_read)
+		if(event_sequence == last_request_read)
 		{
 			error = (xcb_generic_error_t *) event;
 			dpy->xcb->next_event = NULL;
@@ -702,8 +793,7 @@ Status _XReply(Display *dpy, xReply *rep, int extra, Bool discard)
 
 	/* it's not an error, but we don't have a reply, so it's an I/O
 	 * error. */
-	if(!reply)
-	{
+	if(!reply) {
 		_XIOError(dpy);
 		return 0;
 	}
@@ -757,3 +847,30 @@ void _XEatData(Display *dpy, unsigned long n)
 	dpy->xcb->reply_consumed += n;
 	_XFreeReplyData(dpy, False);
 }
+
+/*
+ * Read and discard "n" 32-bit words of data
+ * Matches the units of the length field in X protocol replies, and provides
+ * a single implementation of overflow checking to avoid having to replicate
+ * those checks in every caller.
+ */
+void _XEatDataWords(Display *dpy, unsigned long n)
+{
+	if (n < ((INT_MAX - dpy->xcb->reply_consumed) >> 2))
+		dpy->xcb->reply_consumed += (n << 2);
+	else
+		/* Overflow would happen, so just eat the rest of the reply */
+		dpy->xcb->reply_consumed = dpy->xcb->reply_length;
+	_XFreeReplyData(dpy, False);
+}
+
+unsigned long
+_XNextRequest(Display *dpy)
+{
+    /* This will update dpy->request. The assumption is that the next thing
+     * that the application will do is make a request so there's little
+     * overhead.
+     */
+    require_socket(dpy);
+    return NextRequest(dpy);
+}
